# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/20f_draw.pil_drawer.ipynb.

# %% auto 0
__all__ = ['PILDrawer']

# %% ../../nbs/20f_draw.pil_drawer.ipynb 3
from fastcore.all import *
import polvo as pv
import polvo.bbox as pb

# %% ../../nbs/20f_draw.pil_drawer.ipynb 4
class PILDrawer(pv.Visitor):
    def draw(self, record, image=None, kwargs_list=None):
        self._image = image
        self.visit_all(record, kwargs_list=kwargs_list)
        return self._image
    
    def _visit_image_file(self, image_file, **kwargs): self._image = image_file.open(**kwargs)
    def _visit_image_pil(self, image_pil, **kwargs): self._image = image_pil.draw(drawer)
    def _visit_image_arr(self, image_arr, **kwargs): self._image = image_arr.draw(drawer)
    def _visit_bbox(self, bbox, **kwargs): self._image = pb.overlay(bbox=bbox, image=self._image, **kwargs)
    def _visit_bbox_labelled(self, bbox, **kwargs): self._image = pb.overlay_bbox_labelled(bbox=bbox, image=self._image, **kwargs)
    def _visit_obbox(self, obbox, **kwargs): return self._visit_bbox(obbox, **kwargs)
    def _visit_obbox_labelled(self, obbox, **kwargs): return self._visit_bbox_labelled(obbox, **kwargs)
    def _visit_metadata(self, metadata, **kwargs): pass



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/14g_segmask.vis.ipynb.

# %% auto 0
__all__ = ['overlay_array', 'overlay']

# %% ../../nbs/14g_segmask.vis.ipynb 3
from fastcore.all import *
import PIL
import numpy as np
import polvo as pv
import polvo.segmask as ps
import matplotlib as mpl

# %% ../../nbs/14g_segmask.vis.ipynb 4
# def overlay(
#     image:PIL.Image,
#     mask:ps.SegMask,
# #     nclasses:int,
#     fill_background:bool=True,
#     cmap:mpl.colors.Colormap=mpl.colormaps['gist_ncar'],
#     alpha:float=0.5,
# ):
#     mask_arr = np.array(mask.mask)
#     if not len(mask_arr.shape) == 2: raise ValueError(f'Mask bust be 2d, but got {mask.shape}')
    
#     colored_mask = np.zeros((*mask_arr.shape, 3), dtype=np.uint8)
#     for class_idx in np.unique(mask_arr):
#         if not fill_background and mask.class_map.id2name[class_idx] == 'background': continue
#         mask_idxs = mask_arr == class_idx
#         colored_mask[mask_idxs] = np.array(cmap(class_idx/len(mask.class_map))[:3]) * 255

#     return PIL.Image.blend(image, PIL.Image.fromarray(colored_mask), alpha)

# %% ../../nbs/14g_segmask.vis.ipynb 5
def overlay_array(
    image:PIL.Image,
    mask:Union[PIL.Image.Image, np.ndarray],
    nclasses:int,
    ignore_idxs:Sequence=None,
    cmap:mpl.colors.Colormap=mpl.colormaps['gist_ncar'],
    alpha:float=0.5,
):
    mask = np.asarray(mask)
    ignore_idxs = ignore_idxs or []
    if not len(mask.shape) == 2: raise ValueError(f'Mask bust be 2d, but got {mask.shape}')
    
    colored_mask = np.zeros((*mask.shape, 3), dtype=np.uint8)
    for class_idx in np.unique(mask):
        if class_idx in ignore_idxs: continue
        mask_idxs = mask == class_idx
        colored_mask[mask_idxs] = np.array(cmap(class_idx/nclasses)[:3]) * 255

    return PIL.Image.blend(image, PIL.Image.fromarray(colored_mask), alpha)

# %% ../../nbs/14g_segmask.vis.ipynb 7
def overlay(
    image:PIL.Image,
    mask:ps.SegMask,
    fill_background:bool=True,
    ignore_idxs=None,
    cmap:mpl.colors.Colormap=mpl.colormaps['gist_ncar'],
    alpha:float=0.5,
):
    ignore_idxs = ignore_idxs or []
    if not fill_background: ignore_idxs.append(mask.class_map.name2id['background'])
    return overlay_array(image=image, mask=mask.mask, nclasses=len(mask.class_map), ignore_idxs=ignore_idxs, cmap=cmap, alpha=alpha)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/01u_utils.splits.ipynb.

# %% auto 0
__all__ = ['random', 'from_fn']

# %% ../../nbs/01u_utils.splits.ipynb 2
from fastcore.all import *
import random as _random
from collections import defaultdict
from .misc import random_local_seed

# %% ../../nbs/01u_utils.splits.ipynb 3
def random(items, probs, seed=None):
    # Calculate split indexes
    n = len(items)
    p = [int(round(prob * n)) for prob in probs]  # convert percentage to absolute and round
    p[p.index(max(p))] += n - sum(p)  # adjusts the largest split to ensure the total sum matches the length of items
    # Shuffle items with the given seed
    with random_local_seed(seed):
        shuffled = items[:]
        _random.shuffle(shuffled)
    # Create cumulative split indexes and perform the split
    split_points = [sum(p[:i+1]) for i in range(len(p))]
    splits = [shuffled[start:end] for start, end in zip([0] + split_points[:-1], split_points)]
    return splits

# %% ../../nbs/01u_utils.splits.ipynb 5
def from_fn(items, fn):
    "`fn` should return the index for each subset"
    splits = defaultdict(list)
    for item in items: splits[fn(item)].append(item)
    return list(zip(*sorted(splits.items())))[1]



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/01t_utils.misc.ipynb.

# %% auto 0
__all__ = ['flatten', 'sort_quadrilateral', 'kwargs_grid', 'safe_zip', 'skip_error', 'Cache', 'transparent_cmap', 'partial',
           'random_local_seed', 'random_color']

# %% ../../nbs/01t_utils.misc.ipynb 2
from fastcore.all import *
import numbers
import random
import numpy as np
import polvo as pv

from contextlib import contextmanager
from PIL import Image

# %% ../../nbs/01t_utils.misc.ipynb 3
def flatten(x):
    "From https://stackoverflow.com/a/2158532/6772672"
    if isinstance(x, Iterable) and not isinstance(x, (str, bytes)): 
        return [a for i in x for a in flatten(i)]
    else: return [x]

# %% ../../nbs/01t_utils.misc.ipynb 5
def sort_quadrilateral(points):
    ysorted_idxs = np.argsort(points[:, 1])
    top_idxs, bottom_idxs = ysorted_idxs[:2], ysorted_idxs[2:]
    xsorted_top_idxs = top_idxs[np.argsort(points[top_idxs][:, 0])]
    xreversesorted_bottom_idxs = bottom_idxs[np.argsort(points[bottom_idxs][:, 0])][::-1]
    # Combine the sorted points in the order: [top-left, top-right, bottom-right, bottom-left]
    sorted_points = np.vstack((points[xsorted_top_idxs], points[xreversesorted_bottom_idxs]))
    sorted_idxs = np.concatenate((xsorted_top_idxs, xreversesorted_bottom_idxs))
    return sorted_points, sorted_idxs

# %% ../../nbs/01t_utils.misc.ipynb 8
def kwargs_grid(**kwargs):
    "Returns a generator with all combinations of kwargs"
    return (dict(zip(kwargs.keys(), v)) for v in itertools.product(*kwargs.values()))

# %% ../../nbs/01t_utils.misc.ipynb 10
@functools.wraps(zip)
def safe_zip(*args, **kwargs):
    if len(set(map(len, args))) not in (0, 1):
        raise ValueError(f'All elements should have the same size, but got {[len(x) for x in args]}')
    return zip(*args, **kwargs)

# %% ../../nbs/01t_utils.misc.ipynb 13
class skip_error:
    def __init__(self, fn, log=True):
        'Returns the error instead of raising it.'
        store_attr()
        
    def __call__(self, *args, **kwargs):
        try: return self.fn(*args, **kwargs)
        except Exception as e: 
            if self.log: print(f'{e} {(args, kwargs)}')
            return e

# %% ../../nbs/01t_utils.misc.ipynb 15
class Cache:
    def __init__(self, func, save_dir=None, debug=False):
        self.func = func
        self.save_dir = Path(save_dir or Path.home()/'.cache/polvo')
        self.debug = debug
        
    def get(self, name, use_cache=True, **func_kwargs):
        path = self.save_dir/name
        try: 
            if use_cache: 
                o = pv.open_pickle(path)
                if self.debug: print('Using cache.')
                return o
        except FileNotFoundError: 
            pass
        
        if self.debug: print('Not using cache.')
        x = self.func(**func_kwargs)
        pv.mkdir(path.parent, exist_ok=True)
        pv.save_pickle(x, path)
        return x
    
    def list_cached(self, **kwargs):
        return pv.get_files(self.save_dir, **kwargs)

# %% ../../nbs/01t_utils.misc.ipynb 18
def transparent_cmap():
    from matplotlib.colors import LinearSegmentedColormap
    colors = [(0, 0, 0, 0) for _ in range(256)]
    return LinearSegmentedColormap.from_list('transparent_cmap', colors, N=256)

# %% ../../nbs/01t_utils.misc.ipynb 23
class partial:
    """Wraps functools.partial, same functionality.
    Modifies the original partial `__repr__` and `__str__`.
    """

    def __init__(self, func, *args, **kwargs):
        self._partial = functools.partial(func, *args, **kwargs)

    def __call__(self, *args, **kwargs): return self._partial(*args, **kwargs)

    def __str__(self):
        name = self._partial.func.__name__
        partial_str = str(self._partial)
        return re.sub(r"<.+>", name, partial_str)

    def __repr__(self): return str(self)

# %% ../../nbs/01t_utils.misc.ipynb 24
@contextmanager
def random_local_seed(seed):
    state = random.getstate()
    random.seed(seed)
    try: yield
    finally: random.setstate(state)

# %% ../../nbs/01t_utils.misc.ipynb 25
def random_color(): return tuple(((np.random.random(3)*0.6 + 0.4)*255).astype(int))



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/01v_utils.wandb.ipynb.

# %% auto 0
__all__ = ['WandbRun', 'wandb_upload']

# %% ../../nbs/01v_utils.wandb.ipynb 2
from fastcore.all import *
import polvo as pv

# %% ../../nbs/01v_utils.wandb.ipynb 3
class WandbRun:
    def __init__(self, run=None, **kwargs):
        "Context manager that finishes run when exiting"
        import wandb
        self.run = run or wandb.init(**kwargs)
    def __enter__(self): return self.run
    def __exit__(self, exc_type, exc_value, traceback): self.run.finish()

# %% ../../nbs/01v_utils.wandb.ipynb 4
def wandb_upload(artifact_name:str, *path:str, project=None, type='dataset', run=None):
    "Uploads files or dir to wandb"
    import wandb
    run = run or wandb.init(project=project, job_type="data-upload")
    artifact = wandb.Artifact(artifact_name, type=type)
    
    def _add_file(p): artifact.add_file(p, p)
    for p in path:
        if Path(p).is_file(): _add_file(p)
        else:
            for f in pv.get_files(p): _add_file(f)

    run.log_artifact(artifact)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/17c_visitor.core.ipynb.

# %% auto 0
__all__ = ['Visitor']

# %% ../../nbs/17c_visitor.core.ipynb 3
from fastcore.all import *

# %% ../../nbs/17c_visitor.core.ipynb 4
class Visitor:
    def _visit_image_file(self, image_file, **kwargs): raise NotImplementedError
    def _visit_image_arr(self, image_arr, **kwargs): raise NotImplementedError
    def _visit_image_pil(self, image_pil, **kwargs): raise NotImplementedError
    def _visit_bbox(self, bbox, **kwargs): raise NotImplementedError
    def _visit_bbox_labelled(self, bbox, **kwargs): raise NotImplementedError
    def _visit_obbox(self, bbox, **kwargs): raise NotImplementedError
    def _visit_obbox_labelled(self, obbox, **kwargs): raise NotImplementedError
    def _visit_keypoints(self, keypoints, **kwargs): raise NotImplementedError
    def _visit_segmask(self, segmask, **kwargs): raise NotImplementedError
    def _visit_metadata(self, metadata, **kwargs): raise NotImplementedError
        
    # separating the drawing functions from its call allows us to pass custom arguments to each item
    def collect_accept_fns(self, record):
        return [item.accept_visit(self) for item in record]
    
    def execute_accept_fns(self, fns, kwargs_list=None):
        kwargs_list = kwargs_list or [{} for _ in fns]
        for fn, kwargs in zip(fns, kwargs_list): fn(**kwargs)
        
    def visit_all(self, record, kwargs_list=None):
        fns = self.collect_accept_fns(record)
        return self.execute_accept_fns(fns, kwargs_list=kwargs_list)
    
    def visit_image_file(self, image_file, **kwargs): return partial(self._visit_image_file, image_file, **kwargs)
    def visit_image_arr(self, image_arr, **kwargs): return partial(self._visit_image_arr, image_arr, **kwargs)
    def visit_image_pil(self, image_pil, **kwargs): return partial(self._visit_image_pil, image_pil, **kwargs)
    def visit_bbox(self, bbox, **kwargs): return partial(self._visit_bbox, bbox, **kwargs)
    def visit_bbox_labelled(self, bbox, **kwargs): return partial(self._visit_bbox_labelled, bbox, **kwargs)
    def visit_obbox(self, bbox, **kwargs): return partial(self._visit_obbox, bbox, **kwargs)
    def visit_obbox_labelled(self, bbox, **kwargs): return partial(self._visit_obbox_labelled, bbox, **kwargs)
    def visit_keypoints(self, keypoints, **kwargs): return partial(self._visit_keypoints, keypoints, **kwargs)
    def visit_segmask(self, segmax, **kwargs): return partial(self._visit_segmask, segmax, **kwargs)
    def visit_metadata(self, metadata, **kwargs): return partial(self._visit_metadata, metadata, **kwargs)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/03c_data.core.ipynb.

# %% auto 0
__all__ = ['Component', 'Label', 'ClassMap', 'Metadata']

# %% ../../nbs/03c_data.core.ipynb 2
from fastcore.all import *
from collections import UserDict
import polvo as pv

# %% ../../nbs/03c_data.core.ipynb 3
# Still needs to be figured out
class Component:
    ...

# %% ../../nbs/03c_data.core.ipynb 4
class Label:
    def __init__(self, id, name): store_attr()
    def __repr__(self): return f'{self.id}: {self.name}'
    def __eq__(self, other): 
        return isinstance(other, Label) and (self.id, self.name) == (other.id, other.name)
    def __hash__(self): return hash((self.id, self.name))

# %% ../../nbs/03c_data.core.ipynb 6
class ClassMap:
    def __init__(self, *labels): 
        self.labels = labels
        self.id2name = {l.id: l.name for l in labels}
        self.name2id = {l.name: l.id for l in labels}
        self.id2label = {l.id: l for l in labels}
        self.name2label = {l.name: l for l in labels}
        
    def __len__(self): return len(self.labels)
    def __repr__(self): return f'<{self.__class__.__name__} {self.name2id}>'

# %% ../../nbs/03c_data.core.ipynb 7
class Metadata(UserDict):
    def accept_visit(self, visitor): return visitor.visit_metadata(self)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/25h_converter.coco.ipynb.

# %% auto 0
__all__ = ['COCO']

# %% ../../nbs/25h_converter.coco.ipynb 3
from fastcore.all import *
import cv2
import shutil
import numpy as np
import polvo as pv

# %% ../../nbs/25h_converter.coco.ipynb 4
class COCO(pv.Visitor):
    def _initial_json(self):
        return {
            'info': {'year': '', 'version': ''},
            'licenses': [{'id': 0, 'url': '', 'name': ''}],
            'categories': [],
            'images': [],
            'annotations': []}
    
    def convert_record(self, record, image_dir, ann_dir):
        ...
    
    def convert_records(self, save_dir, records, n_workers=defaults.cpus):
        ...
#         image_dir = pv.mkdir(Path(save_dir)/'images')
#         ann_dir = pv.mkdir(Path(save_dir)/'labels')
        # Parallel might be problematic with current implementation
        # it might get wrong indices when doing len(self._json['images'])
#         parallel(pv.partial(self.convert_record, image_dir=image_dir, ann_dir=ann_dir),
#                  records, progress=pv.pbar, n_workers=n_workers)

        
    def dataset_json(self, records, dataset_class_map):
        self._dataset_class_map = dataset_class_map
        self._json = self._initial_json()
        self._add_categories(dataset_class_map)
        for record in records: self.record_json(record)
        return self._json
    
    def _add_categories(self, class_map):
        for i, name in class_map.id2name.items(): 
            self._json['categories'].append({'id': i, 'name': name, 'supercategory': ''})
        
    def record_json(self, record):
        self._labels, self._bboxes = [], []
        self.visit_all(record)
        
    def _visit_image_file(self, image_file): 
        self._w,self._h = pv.image_size(image_file)
        self._image_file = image_file
        self._image_id = len(self._json['images'])+1
        self._json['images'].append({
            'id': self._image_id,
            'license': 0,
            'file_name': self._image_file.name,
            'height': self._h,
            'width': self._w,
        })
    
    def _visit_bbox_labelled(self, bbox):
        annotation_id = len(self._json['annotations'])
        self._json['annotations'].append({
            'id': annotation_id,
            'image_id': self._image_id,
            'category_id': self._dataset_class_map.name2id[bbox.label.name],
            'bbox': bbox.xyxy.astype(int).tolist(),
            'area': int(bbox.area),
            'segmentation': [bbox.flat.astype(int).tolist()],
            'iscrowd': 0
        })

    def _visit_obbox_labelled(self, obbox):
        annotation_id = len(self._json['annotations'])
        self._json['annotations'].append({
            'id': annotation_id,
            'image_id': self._image_id,
            'category_id': self._dataset_class_map.name2id[obbox.label.name],
            'bbox': obbox.xyxy.astype(int).tolist(),
            'area': int(obbox.area),
            'segmentation': [obbox.flat.astype(int).tolist()],
            'iscrowd': 0
        })
        
    def _visit_segmask(self, segmask):
        contours = segmask.to_contours()
        for label, contour in contours.items():
            for c in contour:
                dataset_catid = self._dataset_class_map.name2id[label]
                self._json['annotations'].append(self._contour_to_annotation(c, dataset_catid))

    def _contour_to_annotation(self, contour, category_id):
        assert contour.shape[1] == 1 #i'm not sure i can just ignore the 2nd dim
        contour = contour[:, 0, :]
        # segmentation
        polygon = contour.ravel().tolist()
        # bbox
        xmax, ymax = contour.max(axis=0).tolist()
        xmin, ymin = contour.min(axis=0).tolist()
        bbox = [xmin, ymin, xmax - xmin, ymax - ymin]

        annotation_id = len(self._json['annotations'])
        return {
            'id': annotation_id,
            'image_id': self._image_id,
            "category_id": category_id,
            "segmentation": [polygon],
            "bbox": bbox,
            "iscrowd": 0,
        }

# %% ../../nbs/25h_converter.coco.ipynb 10
# temporary
import cv2



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/14c_segmask.core.ipynb.

# %% auto 0
__all__ = ['Polygon', 'PolygonLabeled', 'SegMask']

# %% ../../nbs/14c_segmask.core.ipynb 3
from fastcore.all import *
import cv2
import numpy as np
import polvo as pv

# %% ../../nbs/14c_segmask.core.ipynb 4
# To test, do a mask to_polygons and from_polygons
class Polygon:
    def __init__(self, points):
        self.points = np.array(points)
    def __repr__(self): return f"<{self.__class__.__name__} ({self.points})>"
        
    @property
    def xy(self): return self.points
        
    @classmethod
    def from_xy(cls, points): return cls(points)
    @classmethod
    def from_flat(cls, points): return cls(np.reshape(points, (-1, 2)))

# %% ../../nbs/14c_segmask.core.ipynb 5
class PolygonLabeled(GetAttr):
    _default='polygon'
    def __init__(self, polygon, label): store_attr()
    def __repr__(self): return f'{self.label} | {self.polygon}'

# %% ../../nbs/14c_segmask.core.ipynb 7
class SegMask:
    def __init__(self, mask, class_map):
        self.mask = mask
        self.class_map = class_map
       
    def accept_visit(self, visitor): return visitor.visit_segmask(self)
    def show(self): return pv.show_image(self.mask)
    
    # TODO: Refactor to_contours, most of it's logic is the same as `to_polygons`.
    # Maybe we can substitue it completely for to_polygons and update it's uses accordingly (COCOExporter)
    def to_contours(self, method=cv2.CHAIN_APPROX_SIMPLE, mode=cv2.RETR_EXTERNAL):
        arr = self.mask.squeeze().astype(np.uint8)
        contours = {}
        for id_ in set(np.unique(arr)):
            label = self.class_map.id2name[id_]
            if label == 'background': continue
            mask = (arr == id_).astype(np.uint8)
            contours[label] = cv2.findContours(mask, mode, method)[0]
        return contours
    
    def to_polygons(self):
        arr = self.mask.squeeze().astype(np.uint8)
        polygons = []
        for id_ in set(np.unique(arr)):
            label = self.class_map.id2label[id_]
            if label.name == 'background': continue
            mask = (arr == id_).astype(np.uint8)
            for contour in cv2.findContours(mask, mode, method)[0]:
                polygon = Polygon.from_flat(contour.flatten())
                polygons.append(PolygonLabeled(polygon, label))
        return polygons
    
    @classmethod
    def from_polygons(cls, polygons: Sequence[PolygonLabeled], width, height, background_id=0):
        mask_arr = np.zeros((height, width), dtype=np.uint8)
        labels = set((pv.Label(background_id, 'background'),))
        for poly in polygons:
            labels.add(poly.label)
            points = poly.xy.astype(np.int32)
            points = points.reshape((-1, 1, 2))  # Required shape for fillPoly
            cv2.fillPoly(mask_arr, [points], color=poly.label.id)
        return cls(mask_arr, pv.ClassMap(*labels))

    def __repr__(self):
        return f"<{self.__class__.__name__}\n{self.mask}>"



from .logging import *
from .script import *
from .io import *
from .download import *
from .visualization import *
from .misc import *
from .image_size import image_size

from polvo.utils import splits



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/03f_data.transforms.ipynb.

# %% auto 0
__all__ = ['np2pil', 'pil2np', 'AlignOrientation']

# %% ../../nbs/03f_data.transforms.ipynb 2
from fastcore.all import *
from PIL import Image
import numpy as np
import polvo as pv

# %% ../../nbs/03f_data.transforms.ipynb 3
def np2pil(image): return Image.fromarray(image)

# %% ../../nbs/03f_data.transforms.ipynb 4
def pil2np(image): return np.array(image)

# %% ../../nbs/03f_data.transforms.ipynb 5
class AlignOrientation():
    def __init__(self, orientation='vertical', **kwargs):
        "Keep all images in same orientation"
        self.orientation = orientation
        
    def transform(self, x:np.ndarray):
        h, w = x.shape[:2]
        if (w>h and self.orientation=='vertical') or (h>w and self.orientation=='horizontal'):
            self._rotated = True
            return np.rot90(x)
        self._rotated = False
        return x
    
    def reverse(self, x:np.array):
        if self._rotated: return np.rot90(x)
        return x



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/10e_bbox.vis.ipynb.

# %% auto 0
__all__ = ['overlay', 'overlay_label', 'overlay_bbox_labelled']

# %% ../../nbs/10e_bbox.vis.ipynb 4
from fastcore.all import *
import PIL
import numpy as np
import polvo as pv
from typing import Tuple
from PIL import Image, ImageFont
from PIL.ImageDraw import ImageDraw
from . import BBox, OBBox, BBoxLabeled, OBBoxLabeled

# %% ../../nbs/10e_bbox.vis.ipynb 5
def _random_color(): return tuple(((np.random.random(3)*0.6 + 0.4)*255).astype(int))

# %% ../../nbs/10e_bbox.vis.ipynb 6
def overlay(image:PIL.Image, bbox:BBox, color:Tuple[int,int,int]=None):
    """Draws a box on an image with a given color.
    # Arguments
        image     : The image to draw on.
        box       : A list of 4 elements (x1, y1, x2, y2).
        color     : The color of the box.
    """
    draw = PIL.ImageDraw.Draw(image)
    color = color or _random_color()
    bbox_thickness = int(0.0081*min(image.size) - 0.0078)
    bbox_thickness = np.clip(bbox_thickness, 1, 16)
    
    for i in range(4): draw.line(xy=(tuple(bbox.points[i]), tuple(bbox.points[(i+1)%4])), fill=color, width=bbox_thickness)
    return image

# %% ../../nbs/10e_bbox.vis.ipynb 10
def _default_font(scale=1.0):
    return ImageFont.truetype("DejaVuSans.ttf", size=int(10*scale))  # Generic sans-serif
#     except IOError: return ImageFont.load_default() 

# %% ../../nbs/10e_bbox.vis.ipynb 11
def overlay_label(image, label, x, y, color=None, text_scale=1.0, font=None, padding=0):
    color = color or _random_color()
#     font = font or ImageFont.load_default()
    font = font or _default_font(text_scale)
    draw = PIL.ImageDraw.Draw(image)
    # Calculate text size (try/except because font.getsize was deprecated in PIL)
    try:
        x1,y1, x2,y2 = font.getbbox(label)
        text_width, text_height = x2-x1, y2-y1
    except AttributeError:
        text_width, text_height = font.getsize(label)
        
    # Calculate box coordinates with padding
    if (y - text_height - padding) > 0:
        box_pt1, box_pt2 = (x, y+padding), (x+text_width+padding, y-text_height-padding)
        label_pt = (box_pt1[0], box_pt1[1]-text_height)
    else:
        box_pt1, box_pt2 = (x, y+text_height+ padding*2), (x+text_width+padding, y+padding)
        label_pt = (box_pt1[0], box_pt1[1])

    draw.rectangle([box_pt1, box_pt2], fill=color)
    draw.text(label_pt, label, font=font, fill=(240, 240, 240))
    return image

# %% ../../nbs/10e_bbox.vis.ipynb 13
@delegates(overlay_label)
def overlay_bbox_labelled(image, bbox, color=None, **kwargs):
    color = color or _random_color()
    image = overlay(image, bbox, color=color)
    x,y = bbox.xyxyxyxy[0]
    return overlay_label(image, bbox.label.name, x,y, color=color, **kwargs)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/25f_converter.yolo.ipynb.

# %% auto 0
__all__ = ['YOLO']

# %% ../../nbs/25f_converter.yolo.ipynb 3
from fastcore.all import *
import shutil
import polvo as pv

# %% ../../nbs/25f_converter.yolo.ipynb 4
class YOLO(pv.Visitor):
    def convert_dataset(self, save_dir, classes, train=None, valid=None, test=None, n_workers=defaults.cpus):
        save_dir = Path(save_dir)
        dirs = {}
        if train: 
            train_dir = dirs['train_dir'] = pv.mkdir(save_dir/'train')
            self.convert_records(train_dir, train, n_workers=n_workers)
        if valid:
            valid_dir = dirs['valid_dir'] = pv.mkdir(save_dir/'valid')
            self.convert_records(valid_dir, valid, n_workers=n_workers)
        if test: 
            test_dir = dirs['test_dir'] = pv.mkdir(save_dir/'test')
            self.convert_records(test_dir, test, n_workers=n_workers)
            
        yolo_yaml = self.yolo_yaml(classes, save_dir, **dirs)
        pv.save_txt(yolo_yaml, save_dir/'dataset.yaml')
        
    def convert_records(self, save_dir, records, n_workers=defaults.cpus):
        image_dir = pv.mkdir(Path(save_dir)/'images')
        ann_dir = pv.mkdir(Path(save_dir)/'labels')
        parallel(pv.partial(self.convert_record, image_dir=image_dir, ann_dir=ann_dir),
                 records, progress=pv.pbar, n_workers=n_workers)
        
    def convert_record(self, record, image_dir, ann_dir):
        self._labels, self._bboxes = [], []
        self.visit_all(record)
        lines = [' '.join(o) for o in pv.safe_zip(self._labels, self._bboxes)]
        pv.save_txt('\n'.join(lines), ann_dir/self._image_file.with_suffix('.txt').name)
        shutil.copy(self._image_file.absolute(), image_dir/self._image_file.name)
        
    def yolo_yaml(self, class_map, save_dir, train_dir=None, valid_dir=None, test_dir=None):
        classes = '\n'.join([f'  {k}: {v}' for k,v in class_map.items()])
        train_str = f"train: {train_dir.relative_to(save_dir)}\n" if train_dir is not None else ""
        valid_str = f"val: {valid_dir.relative_to(save_dir)}\n" if valid_dir is not None else ""
        test_str = f"test: {test_dir.relative_to(save_dir)}\n" if test_dir is not None else ""
        return (
            f"path: {save_dir.absolute()}\n"
            f"{train_str}"
            f"{valid_str}"
            f"{test_str}"
            f"names:\n"
            f"{classes}"
        )
    
    def _visit_image_file(self, image_file): 
        self._w,self._h = pv.image_size(image_file)
        self._image_file = image_file
    
    def _visit_bbox_labelled(self, bbox):
        self._labels.append(str(bbox.label.id))
        points = bbox.normalized_points(self._w,self._h).flatten()
        self._bboxes.append(' '.join(map(str, points)))



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/12c_keypoint.core.ipynb.

# %% auto 0
__all__ = ['Keypoints']

# %% ../../nbs/12c_keypoint.core.ipynb 3
from fastcore.all import *
import numpy as np
import polvo as pv

# %% ../../nbs/12c_keypoint.core.ipynb 4
class Keypoints:
    def __init__(self, points):
        if points.ndim != 2: raise ValueError(f'points should be 2-dim [(x1, y1), (x2, y2), ...] but got {points.ndim}-dim')
        self.points = points
        
    @property
    def xy_points(self): return self.points
        
    def accept_visit(self, visitor): return visitor.visit_keypoints(self)
    
    def normalized(self, imagew, imageh):
        scale = np.array([imagew, imageh])
        return self.points / scale
    
    @classmethod
    def from_2d(cls, points): return cls(points)
    @classmethod
    def from_flat(cls, points): return cls(np.reshape(points, (-1, 2)))
    
    def __repr__(self):
        return f"<{self.__class__.__name__} {self.points}>"



# Autogenerated by nbdev

d = { 'settings': { 'branch': 'master',
                'doc_baseurl': '/polvo/',
                'doc_host': 'https://lgvaz.github.io',
                'git_url': 'https://github.com/lgvaz/polvo/',
                'lib_path': 'polvo'},
  'syms': { 'polvo.bbox.core': { 'polvo.bbox.core.BBox': ('bbox.core.html#bbox', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core.BBox.__eq__': ('bbox.core.html#bbox.__eq__', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core.BBox.__repr__': ('bbox.core.html#bbox.__repr__', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core.BBox.accept_visit': ('bbox.core.html#bbox.accept_visit', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core.BBox.from_relative_xcycwh': ( 'bbox.core.html#bbox.from_relative_xcycwh',
                                                                                'polvo/bbox/core.py'),
                                 'polvo.bbox.core.BBox.from_rle': ('bbox.core.html#bbox.from_rle', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core.BBox.from_xycenter_wh': ('bbox.core.html#bbox.from_xycenter_wh', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core.BBox.from_xywh': ('bbox.core.html#bbox.from_xywh', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core.BBox.from_xyxy': ('bbox.core.html#bbox.from_xyxy', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core.BBox.normalized_xycenter_wh': ( 'bbox.core.html#bbox.normalized_xycenter_wh',
                                                                                  'polvo/bbox/core.py'),
                                 'polvo.bbox.core.BBox.relative_xcycwh': ('bbox.core.html#bbox.relative_xcycwh', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core.BBoxLabeled': ('bbox.core.html#bboxlabeled', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core.BBoxLabeled.accept_visit': ( 'bbox.core.html#bboxlabeled.accept_visit',
                                                                               'polvo/bbox/core.py'),
                                 'polvo.bbox.core.OBBox': ('bbox.core.html#obbox', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core.OBBox.__repr__': ('bbox.core.html#obbox.__repr__', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core.OBBox.accept_visit': ('bbox.core.html#obbox.accept_visit', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core.OBBox.from_clockwise': ('bbox.core.html#obbox.from_clockwise', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core.OBBoxLabeled': ('bbox.core.html#obboxlabeled', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core.OBBoxLabeled.accept_visit': ( 'bbox.core.html#obboxlabeled.accept_visit',
                                                                                'polvo/bbox/core.py'),
                                 'polvo.bbox.core._BBox': ('bbox.core.html#_bbox', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core._BBox.__init__': ('bbox.core.html#_bbox.__init__', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core._BBox.area': ('bbox.core.html#_bbox.area', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core._BBox.flat': ('bbox.core.html#_bbox.flat', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core._BBox.from_flat': ('bbox.core.html#_bbox.from_flat', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core._BBox.from_points': ('bbox.core.html#_bbox.from_points', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core._BBox.height': ('bbox.core.html#_bbox.height', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core._BBox.normalized_points': ( 'bbox.core.html#_bbox.normalized_points',
                                                                              'polvo/bbox/core.py'),
                                 'polvo.bbox.core._BBox.width': ('bbox.core.html#_bbox.width', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core._BBox.xmax': ('bbox.core.html#_bbox.xmax', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core._BBox.xmin': ('bbox.core.html#_bbox.xmin', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core._BBox.xycenter_wh': ('bbox.core.html#_bbox.xycenter_wh', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core._BBox.xywh': ('bbox.core.html#_bbox.xywh', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core._BBox.xyxy': ('bbox.core.html#_bbox.xyxy', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core._BBox.xyxyxyxy': ('bbox.core.html#_bbox.xyxyxyxy', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core._BBox.ymax': ('bbox.core.html#_bbox.ymax', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core._BBox.ymin': ('bbox.core.html#_bbox.ymin', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core._BBox.yxyx': ('bbox.core.html#_bbox.yxyx', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core._BBoxLabeled': ('bbox.core.html#_bboxlabeled', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core._BBoxLabeled.__init__': ('bbox.core.html#_bboxlabeled.__init__', 'polvo/bbox/core.py'),
                                 'polvo.bbox.core._BBoxLabeled.__repr__': ('bbox.core.html#_bboxlabeled.__repr__', 'polvo/bbox/core.py')},
            'polvo.bbox.vis': { 'polvo.bbox.vis._default_font': ('bbox.vis.html#_default_font', 'polvo/bbox/vis.py'),
                                'polvo.bbox.vis._random_color': ('bbox.vis.html#_random_color', 'polvo/bbox/vis.py'),
                                'polvo.bbox.vis.overlay': ('bbox.vis.html#overlay', 'polvo/bbox/vis.py'),
                                'polvo.bbox.vis.overlay_bbox_labelled': ('bbox.vis.html#overlay_bbox_labelled', 'polvo/bbox/vis.py'),
                                'polvo.bbox.vis.overlay_label': ('bbox.vis.html#overlay_label', 'polvo/bbox/vis.py')},
            'polvo.classification.baseline.baseline': { 'polvo.classification.baseline.baseline.FastaiMetricWrapper': ( 'classification.baseline.baseline.html#fastaimetricwrapper',
                                                                                                                        'polvo/classification/baseline/baseline.py'),
                                                        'polvo.classification.baseline.baseline.FastaiMetricWrapper.__init__': ( 'classification.baseline.baseline.html#fastaimetricwrapper.__init__',
                                                                                                                                 'polvo/classification/baseline/baseline.py'),
                                                        'polvo.classification.baseline.baseline.FastaiMetricWrapper.accumulate': ( 'classification.baseline.baseline.html#fastaimetricwrapper.accumulate',
                                                                                                                                   'polvo/classification/baseline/baseline.py'),
                                                        'polvo.classification.baseline.baseline.calculate_metrics': ( 'classification.baseline.baseline.html#calculate_metrics',
                                                                                                                      'polvo/classification/baseline/baseline.py'),
                                                        'polvo.classification.baseline.baseline.fixed_probs': ( 'classification.baseline.baseline.html#fixed_probs',
                                                                                                                'polvo/classification/baseline/baseline.py'),
                                                        'polvo.classification.baseline.baseline.random_probs': ( 'classification.baseline.baseline.html#random_probs',
                                                                                                                 'polvo/classification/baseline/baseline.py')},
            'polvo.classification.explorer': { 'polvo.classification.explorer.DataExplorer': ( 'classification.explorer.html#dataexplorer',
                                                                                               'polvo/classification/explorer.py'),
                                               'polvo.classification.explorer.DataExplorer.__init__': ( 'classification.explorer.html#dataexplorer.__init__',
                                                                                                        'polvo/classification/explorer.py'),
                                               'polvo.classification.explorer.DataExplorer._group_y': ( 'classification.explorer.html#dataexplorer._group_y',
                                                                                                        'polvo/classification/explorer.py'),
                                               'polvo.classification.explorer.DataExplorer.most_common': ( 'classification.explorer.html#dataexplorer.most_common',
                                                                                                           'polvo/classification/explorer.py'),
                                               'polvo.classification.explorer.DataExplorer.show_label': ( 'classification.explorer.html#dataexplorer.show_label',
                                                                                                          'polvo/classification/explorer.py'),
                                               'polvo.classification.explorer.DataExplorer.show_label_id': ( 'classification.explorer.html#dataexplorer.show_label_id',
                                                                                                             'polvo/classification/explorer.py'),
                                               'polvo.classification.explorer.DataExplorer.show_most_common': ( 'classification.explorer.html#dataexplorer.show_most_common',
                                                                                                                'polvo/classification/explorer.py')},
            'polvo.converter.coco': { 'polvo.converter.coco.COCO': ('converter.coco.html#coco', 'polvo/converter/coco.py'),
                                      'polvo.converter.coco.COCO._add_categories': ( 'converter.coco.html#coco._add_categories',
                                                                                     'polvo/converter/coco.py'),
                                      'polvo.converter.coco.COCO._contour_to_annotation': ( 'converter.coco.html#coco._contour_to_annotation',
                                                                                            'polvo/converter/coco.py'),
                                      'polvo.converter.coco.COCO._initial_json': ( 'converter.coco.html#coco._initial_json',
                                                                                   'polvo/converter/coco.py'),
                                      'polvo.converter.coco.COCO._visit_bbox_labelled': ( 'converter.coco.html#coco._visit_bbox_labelled',
                                                                                          'polvo/converter/coco.py'),
                                      'polvo.converter.coco.COCO._visit_image_file': ( 'converter.coco.html#coco._visit_image_file',
                                                                                       'polvo/converter/coco.py'),
                                      'polvo.converter.coco.COCO._visit_obbox_labelled': ( 'converter.coco.html#coco._visit_obbox_labelled',
                                                                                           'polvo/converter/coco.py'),
                                      'polvo.converter.coco.COCO._visit_segmask': ( 'converter.coco.html#coco._visit_segmask',
                                                                                    'polvo/converter/coco.py'),
                                      'polvo.converter.coco.COCO.convert_record': ( 'converter.coco.html#coco.convert_record',
                                                                                    'polvo/converter/coco.py'),
                                      'polvo.converter.coco.COCO.convert_records': ( 'converter.coco.html#coco.convert_records',
                                                                                     'polvo/converter/coco.py'),
                                      'polvo.converter.coco.COCO.dataset_json': ( 'converter.coco.html#coco.dataset_json',
                                                                                  'polvo/converter/coco.py'),
                                      'polvo.converter.coco.COCO.record_json': ( 'converter.coco.html#coco.record_json',
                                                                                 'polvo/converter/coco.py')},
            'polvo.converter.yolo': { 'polvo.converter.yolo.YOLO': ('converter.yolo.html#yolo', 'polvo/converter/yolo.py'),
                                      'polvo.converter.yolo.YOLO._visit_bbox_labelled': ( 'converter.yolo.html#yolo._visit_bbox_labelled',
                                                                                          'polvo/converter/yolo.py'),
                                      'polvo.converter.yolo.YOLO._visit_image_file': ( 'converter.yolo.html#yolo._visit_image_file',
                                                                                       'polvo/converter/yolo.py'),
                                      'polvo.converter.yolo.YOLO.convert_dataset': ( 'converter.yolo.html#yolo.convert_dataset',
                                                                                     'polvo/converter/yolo.py'),
                                      'polvo.converter.yolo.YOLO.convert_record': ( 'converter.yolo.html#yolo.convert_record',
                                                                                    'polvo/converter/yolo.py'),
                                      'polvo.converter.yolo.YOLO.convert_records': ( 'converter.yolo.html#yolo.convert_records',
                                                                                     'polvo/converter/yolo.py'),
                                      'polvo.converter.yolo.YOLO.yolo_yaml': ( 'converter.yolo.html#yolo.yolo_yaml',
                                                                               'polvo/converter/yolo.py')},
            'polvo.data.core': { 'polvo.data.core.ClassMap': ('data.core.html#classmap', 'polvo/data/core.py'),
                                 'polvo.data.core.ClassMap.__init__': ('data.core.html#classmap.__init__', 'polvo/data/core.py'),
                                 'polvo.data.core.ClassMap.__len__': ('data.core.html#classmap.__len__', 'polvo/data/core.py'),
                                 'polvo.data.core.ClassMap.__repr__': ('data.core.html#classmap.__repr__', 'polvo/data/core.py'),
                                 'polvo.data.core.Component': ('data.core.html#component', 'polvo/data/core.py'),
                                 'polvo.data.core.Label': ('data.core.html#label', 'polvo/data/core.py'),
                                 'polvo.data.core.Label.__eq__': ('data.core.html#label.__eq__', 'polvo/data/core.py'),
                                 'polvo.data.core.Label.__hash__': ('data.core.html#label.__hash__', 'polvo/data/core.py'),
                                 'polvo.data.core.Label.__init__': ('data.core.html#label.__init__', 'polvo/data/core.py'),
                                 'polvo.data.core.Label.__repr__': ('data.core.html#label.__repr__', 'polvo/data/core.py'),
                                 'polvo.data.core.Metadata': ('data.core.html#metadata', 'polvo/data/core.py'),
                                 'polvo.data.core.Metadata.accept_visit': ('data.core.html#metadata.accept_visit', 'polvo/data/core.py')},
            'polvo.data.transforms': { 'polvo.data.transforms.AlignOrientation': ( 'data.transforms.html#alignorientation',
                                                                                   'polvo/data/transforms.py'),
                                       'polvo.data.transforms.AlignOrientation.__init__': ( 'data.transforms.html#alignorientation.__init__',
                                                                                            'polvo/data/transforms.py'),
                                       'polvo.data.transforms.AlignOrientation.reverse': ( 'data.transforms.html#alignorientation.reverse',
                                                                                           'polvo/data/transforms.py'),
                                       'polvo.data.transforms.AlignOrientation.transform': ( 'data.transforms.html#alignorientation.transform',
                                                                                             'polvo/data/transforms.py'),
                                       'polvo.data.transforms.np2pil': ('data.transforms.html#np2pil', 'polvo/data/transforms.py'),
                                       'polvo.data.transforms.pil2np': ('data.transforms.html#pil2np', 'polvo/data/transforms.py')},
            'polvo.datasets.download': { 'polvo.datasets.download.snacks_tiny': ( 'datasets.download.html#snacks_tiny',
                                                                                  'polvo/datasets/download.py')},
            'polvo.draw.pil_drawer': { 'polvo.draw.pil_drawer.PILDrawer': ('draw.pil_drawer.html#pildrawer', 'polvo/draw/pil_drawer.py'),
                                       'polvo.draw.pil_drawer.PILDrawer._visit_bbox': ( 'draw.pil_drawer.html#pildrawer._visit_bbox',
                                                                                        'polvo/draw/pil_drawer.py'),
                                       'polvo.draw.pil_drawer.PILDrawer._visit_bbox_labelled': ( 'draw.pil_drawer.html#pildrawer._visit_bbox_labelled',
                                                                                                 'polvo/draw/pil_drawer.py'),
                                       'polvo.draw.pil_drawer.PILDrawer._visit_image_arr': ( 'draw.pil_drawer.html#pildrawer._visit_image_arr',
                                                                                             'polvo/draw/pil_drawer.py'),
                                       'polvo.draw.pil_drawer.PILDrawer._visit_image_file': ( 'draw.pil_drawer.html#pildrawer._visit_image_file',
                                                                                              'polvo/draw/pil_drawer.py'),
                                       'polvo.draw.pil_drawer.PILDrawer._visit_image_pil': ( 'draw.pil_drawer.html#pildrawer._visit_image_pil',
                                                                                             'polvo/draw/pil_drawer.py'),
                                       'polvo.draw.pil_drawer.PILDrawer._visit_metadata': ( 'draw.pil_drawer.html#pildrawer._visit_metadata',
                                                                                            'polvo/draw/pil_drawer.py'),
                                       'polvo.draw.pil_drawer.PILDrawer._visit_obbox': ( 'draw.pil_drawer.html#pildrawer._visit_obbox',
                                                                                         'polvo/draw/pil_drawer.py'),
                                       'polvo.draw.pil_drawer.PILDrawer._visit_obbox_labelled': ( 'draw.pil_drawer.html#pildrawer._visit_obbox_labelled',
                                                                                                  'polvo/draw/pil_drawer.py'),
                                       'polvo.draw.pil_drawer.PILDrawer.draw': ( 'draw.pil_drawer.html#pildrawer.draw',
                                                                                 'polvo/draw/pil_drawer.py')},
            'polvo.keypoint.core': { 'polvo.keypoint.core.Keypoints': ('keypoint.core.html#keypoints', 'polvo/keypoint/core.py'),
                                     'polvo.keypoint.core.Keypoints.__init__': ( 'keypoint.core.html#keypoints.__init__',
                                                                                 'polvo/keypoint/core.py'),
                                     'polvo.keypoint.core.Keypoints.__repr__': ( 'keypoint.core.html#keypoints.__repr__',
                                                                                 'polvo/keypoint/core.py'),
                                     'polvo.keypoint.core.Keypoints.accept_visit': ( 'keypoint.core.html#keypoints.accept_visit',
                                                                                     'polvo/keypoint/core.py'),
                                     'polvo.keypoint.core.Keypoints.from_2d': ( 'keypoint.core.html#keypoints.from_2d',
                                                                                'polvo/keypoint/core.py'),
                                     'polvo.keypoint.core.Keypoints.from_flat': ( 'keypoint.core.html#keypoints.from_flat',
                                                                                  'polvo/keypoint/core.py'),
                                     'polvo.keypoint.core.Keypoints.normalized': ( 'keypoint.core.html#keypoints.normalized',
                                                                                   'polvo/keypoint/core.py'),
                                     'polvo.keypoint.core.Keypoints.xy_points': ( 'keypoint.core.html#keypoints.xy_points',
                                                                                  'polvo/keypoint/core.py')},
            'polvo.keypoint.vis': {'polvo.keypoint.vis.overlay': ('keypoints.vis.html#overlay', 'polvo/keypoint/vis.py')},
            'polvo.segmask.core': { 'polvo.segmask.core.Polygon': ('segmask.core.html#polygon', 'polvo/segmask/core.py'),
                                    'polvo.segmask.core.Polygon.__init__': ('segmask.core.html#polygon.__init__', 'polvo/segmask/core.py'),
                                    'polvo.segmask.core.Polygon.__repr__': ('segmask.core.html#polygon.__repr__', 'polvo/segmask/core.py'),
                                    'polvo.segmask.core.Polygon.from_flat': ( 'segmask.core.html#polygon.from_flat',
                                                                              'polvo/segmask/core.py'),
                                    'polvo.segmask.core.Polygon.from_xy': ('segmask.core.html#polygon.from_xy', 'polvo/segmask/core.py'),
                                    'polvo.segmask.core.Polygon.xy': ('segmask.core.html#polygon.xy', 'polvo/segmask/core.py'),
                                    'polvo.segmask.core.PolygonLabeled': ('segmask.core.html#polygonlabeled', 'polvo/segmask/core.py'),
                                    'polvo.segmask.core.PolygonLabeled.__init__': ( 'segmask.core.html#polygonlabeled.__init__',
                                                                                    'polvo/segmask/core.py'),
                                    'polvo.segmask.core.PolygonLabeled.__repr__': ( 'segmask.core.html#polygonlabeled.__repr__',
                                                                                    'polvo/segmask/core.py'),
                                    'polvo.segmask.core.SegMask': ('segmask.core.html#segmask', 'polvo/segmask/core.py'),
                                    'polvo.segmask.core.SegMask.__init__': ('segmask.core.html#segmask.__init__', 'polvo/segmask/core.py'),
                                    'polvo.segmask.core.SegMask.__repr__': ('segmask.core.html#segmask.__repr__', 'polvo/segmask/core.py'),
                                    'polvo.segmask.core.SegMask.accept_visit': ( 'segmask.core.html#segmask.accept_visit',
                                                                                 'polvo/segmask/core.py'),
                                    'polvo.segmask.core.SegMask.from_polygons': ( 'segmask.core.html#segmask.from_polygons',
                                                                                  'polvo/segmask/core.py'),
                                    'polvo.segmask.core.SegMask.show': ('segmask.core.html#segmask.show', 'polvo/segmask/core.py'),
                                    'polvo.segmask.core.SegMask.to_contours': ( 'segmask.core.html#segmask.to_contours',
                                                                                'polvo/segmask/core.py'),
                                    'polvo.segmask.core.SegMask.to_polygons': ( 'segmask.core.html#segmask.to_polygons',
                                                                                'polvo/segmask/core.py')},
            'polvo.segmask.vis': { 'polvo.segmask.vis.overlay': ('segmask.vis.html#overlay', 'polvo/segmask/vis.py'),
                                   'polvo.segmask.vis.overlay_array': ('segmask.vis.html#overlay_array', 'polvo/segmask/vis.py')},
            'polvo.segmentation.vis': { 'polvo.segmentation.vis.overlay_mask': ( 'segmentation.vis.html#overlay_mask',
                                                                                 'polvo/segmentation/vis.py')},
            'polvo.test': {},
            'polvo.utils.download': { 'polvo.utils.download.download': ('utils.download.html#download', 'polvo/utils/download.py'),
                                      'polvo.utils.download.download_and_extract': ( 'utils.download.html#download_and_extract',
                                                                                     'polvo/utils/download.py'),
                                      'polvo.utils.download.download_and_extract_gdrive': ( 'utils.download.html#download_and_extract_gdrive',
                                                                                            'polvo/utils/download.py'),
                                      'polvo.utils.download.download_gdrive': ( 'utils.download.html#download_gdrive',
                                                                                'polvo/utils/download.py'),
                                      'polvo.utils.download.open_image_url': ( 'utils.download.html#open_image_url',
                                                                               'polvo/utils/download.py')},
            'polvo.utils.image_size': { 'polvo.utils.image_size.Image': ('utils.image_size.html#image', 'polvo/utils/image_size.py'),
                                        'polvo.utils.image_size.Image.to_str_json': ( 'utils.image_size.html#image.to_str_json',
                                                                                      'polvo/utils/image_size.py'),
                                        'polvo.utils.image_size.Image.to_str_row': ( 'utils.image_size.html#image.to_str_row',
                                                                                     'polvo/utils/image_size.py'),
                                        'polvo.utils.image_size.Image.to_str_row_verbose': ( 'utils.image_size.html#image.to_str_row_verbose',
                                                                                             'polvo/utils/image_size.py'),
                                        'polvo.utils.image_size.UnknownImageFormat': ( 'utils.image_size.html#unknownimageformat',
                                                                                       'polvo/utils/image_size.py'),
                                        'polvo.utils.image_size.get_image_metadata': ( 'utils.image_size.html#get_image_metadata',
                                                                                       'polvo/utils/image_size.py'),
                                        'polvo.utils.image_size.get_image_metadata_from_bytesio': ( 'utils.image_size.html#get_image_metadata_from_bytesio',
                                                                                                    'polvo/utils/image_size.py'),
                                        'polvo.utils.image_size.image_size': ( 'utils.image_size.html#image_size',
                                                                               'polvo/utils/image_size.py'),
                                        'polvo.utils.image_size.image_size_from_bytesio': ( 'utils.image_size.html#image_size_from_bytesio',
                                                                                            'polvo/utils/image_size.py')},
            'polvo.utils.io': { 'polvo.utils.io.ImageArr': ('utils.io.html#imagearr', 'polvo/utils/io.py'),
                                'polvo.utils.io.ImageArr.__init__': ('utils.io.html#imagearr.__init__', 'polvo/utils/io.py'),
                                'polvo.utils.io.ImageArr.accept_visit': ('utils.io.html#imagearr.accept_visit', 'polvo/utils/io.py'),
                                'polvo.utils.io.ImageArr.draw': ('utils.io.html#imagearr.draw', 'polvo/utils/io.py'),
                                'polvo.utils.io.ImageFile': ('utils.io.html#imagefile', 'polvo/utils/io.py'),
                                'polvo.utils.io.ImageFile.__new__': ('utils.io.html#imagefile.__new__', 'polvo/utils/io.py'),
                                'polvo.utils.io.ImageFile.accept_visit': ('utils.io.html#imagefile.accept_visit', 'polvo/utils/io.py'),
                                'polvo.utils.io.ImageFile.draw': ('utils.io.html#imagefile.draw', 'polvo/utils/io.py'),
                                'polvo.utils.io.ImageFile.open': ('utils.io.html#imagefile.open', 'polvo/utils/io.py'),
                                'polvo.utils.io.ImagePIL': ('utils.io.html#imagepil', 'polvo/utils/io.py'),
                                'polvo.utils.io.ImagePIL.__init__': ('utils.io.html#imagepil.__init__', 'polvo/utils/io.py'),
                                'polvo.utils.io.ImagePIL.accept_visit': ('utils.io.html#imagepil.accept_visit', 'polvo/utils/io.py'),
                                'polvo.utils.io.ImagePIL.draw': ('utils.io.html#imagepil.draw', 'polvo/utils/io.py'),
                                'polvo.utils.io.RenderDict': ('utils.io.html#renderdict', 'polvo/utils/io.py'),
                                'polvo.utils.io.RenderDict.__init__': ('utils.io.html#renderdict.__init__', 'polvo/utils/io.py'),
                                'polvo.utils.io.RenderDict._ipython_display_': ( 'utils.io.html#renderdict._ipython_display_',
                                                                                 'polvo/utils/io.py'),
                                'polvo.utils.io._get_files': ('utils.io.html#_get_files', 'polvo/utils/io.py'),
                                'polvo.utils.io.extract_files': ('utils.io.html#extract_files', 'polvo/utils/io.py'),
                                'polvo.utils.io.get_files': ('utils.io.html#get_files', 'polvo/utils/io.py'),
                                'polvo.utils.io.get_image_files': ('utils.io.html#get_image_files', 'polvo/utils/io.py'),
                                'polvo.utils.io.glob_match': ('utils.io.html#glob_match', 'polvo/utils/io.py'),
                                'polvo.utils.io.mkdir': ('utils.io.html#mkdir', 'polvo/utils/io.py'),
                                'polvo.utils.io.open_dill': ('utils.io.html#open_dill', 'polvo/utils/io.py'),
                                'polvo.utils.io.open_image': ('utils.io.html#open_image', 'polvo/utils/io.py'),
                                'polvo.utils.io.open_json': ('utils.io.html#open_json', 'polvo/utils/io.py'),
                                'polvo.utils.io.open_mask': ('utils.io.html#open_mask', 'polvo/utils/io.py'),
                                'polvo.utils.io.open_pickle': ('utils.io.html#open_pickle', 'polvo/utils/io.py'),
                                'polvo.utils.io.open_txt': ('utils.io.html#open_txt', 'polvo/utils/io.py'),
                                'polvo.utils.io.save_dill': ('utils.io.html#save_dill', 'polvo/utils/io.py'),
                                'polvo.utils.io.save_image': ('utils.io.html#save_image', 'polvo/utils/io.py'),
                                'polvo.utils.io.save_json': ('utils.io.html#save_json', 'polvo/utils/io.py'),
                                'polvo.utils.io.save_pickle': ('utils.io.html#save_pickle', 'polvo/utils/io.py'),
                                'polvo.utils.io.save_txt': ('utils.io.html#save_txt', 'polvo/utils/io.py')},
            'polvo.utils.logging': {'polvo.utils.logging.pbar': ('utils.logging.html#pbar', 'polvo/utils/logging.py')},
            'polvo.utils.misc': { 'polvo.utils.misc.Cache': ('utils.misc.html#cache', 'polvo/utils/misc.py'),
                                  'polvo.utils.misc.Cache.__init__': ('utils.misc.html#cache.__init__', 'polvo/utils/misc.py'),
                                  'polvo.utils.misc.Cache.get': ('utils.misc.html#cache.get', 'polvo/utils/misc.py'),
                                  'polvo.utils.misc.Cache.list_cached': ('utils.misc.html#cache.list_cached', 'polvo/utils/misc.py'),
                                  'polvo.utils.misc.flatten': ('utils.misc.html#flatten', 'polvo/utils/misc.py'),
                                  'polvo.utils.misc.kwargs_grid': ('utils.misc.html#kwargs_grid', 'polvo/utils/misc.py'),
                                  'polvo.utils.misc.partial': ('utils.misc.html#partial', 'polvo/utils/misc.py'),
                                  'polvo.utils.misc.partial.__call__': ('utils.misc.html#partial.__call__', 'polvo/utils/misc.py'),
                                  'polvo.utils.misc.partial.__init__': ('utils.misc.html#partial.__init__', 'polvo/utils/misc.py'),
                                  'polvo.utils.misc.partial.__repr__': ('utils.misc.html#partial.__repr__', 'polvo/utils/misc.py'),
                                  'polvo.utils.misc.partial.__str__': ('utils.misc.html#partial.__str__', 'polvo/utils/misc.py'),
                                  'polvo.utils.misc.random_color': ('utils.misc.html#random_color', 'polvo/utils/misc.py'),
                                  'polvo.utils.misc.random_local_seed': ('utils.misc.html#random_local_seed', 'polvo/utils/misc.py'),
                                  'polvo.utils.misc.safe_zip': ('utils.misc.html#safe_zip', 'polvo/utils/misc.py'),
                                  'polvo.utils.misc.skip_error': ('utils.misc.html#skip_error', 'polvo/utils/misc.py'),
                                  'polvo.utils.misc.skip_error.__call__': ('utils.misc.html#skip_error.__call__', 'polvo/utils/misc.py'),
                                  'polvo.utils.misc.skip_error.__init__': ('utils.misc.html#skip_error.__init__', 'polvo/utils/misc.py'),
                                  'polvo.utils.misc.sort_quadrilateral': ('utils.misc.html#sort_quadrilateral', 'polvo/utils/misc.py'),
                                  'polvo.utils.misc.transparent_cmap': ('utils.misc.html#transparent_cmap', 'polvo/utils/misc.py')},
            'polvo.utils.script': { 'polvo.utils.script.call_parse': ('utils.script.html#call_parse', 'polvo/utils/script.py'),
                                    'polvo.utils.script.test_script': ('utils.script.html#test_script', 'polvo/utils/script.py')},
            'polvo.utils.splits': { 'polvo.utils.splits.from_fn': ('utils.splits.html#from_fn', 'polvo/utils/splits.py'),
                                    'polvo.utils.splits.random': ('utils.splits.html#random', 'polvo/utils/splits.py')},
            'polvo.utils.visualization': { 'polvo.utils.visualization.get_grid': ( 'utils.visualization.html#get_grid',
                                                                                   'polvo/utils/visualization.py'),
                                           'polvo.utils.visualization.get_image_grid': ( 'utils.visualization.html#get_image_grid',
                                                                                         'polvo/utils/visualization.py'),
                                           'polvo.utils.visualization.grid_from_sequence': ( 'utils.visualization.html#grid_from_sequence',
                                                                                             'polvo/utils/visualization.py'),
                                           'polvo.utils.visualization.image_grid': ( 'utils.visualization.html#image_grid',
                                                                                     'polvo/utils/visualization.py'),
                                           'polvo.utils.visualization.image_size': ( 'utils.visualization.html#image_size',
                                                                                     'polvo/utils/visualization.py'),
                                           'polvo.utils.visualization.image_sizes_hist': ( 'utils.visualization.html#image_sizes_hist',
                                                                                           'polvo/utils/visualization.py'),
                                           'polvo.utils.visualization.normalize_axes': ( 'utils.visualization.html#normalize_axes',
                                                                                         'polvo/utils/visualization.py'),
                                           'polvo.utils.visualization.plot_grid': ( 'utils.visualization.html#plot_grid',
                                                                                    'polvo/utils/visualization.py'),
                                           'polvo.utils.visualization.show_grid': ( 'utils.visualization.html#show_grid',
                                                                                    'polvo/utils/visualization.py'),
                                           'polvo.utils.visualization.show_image': ( 'utils.visualization.html#show_image',
                                                                                     'polvo/utils/visualization.py'),
                                           'polvo.utils.visualization.show_images': ( 'utils.visualization.html#show_images',
                                                                                      'polvo/utils/visualization.py')},
            'polvo.utils.wandb': { 'polvo.utils.wandb.WandbRun': ('utils.wandb.html#wandbrun', 'polvo/utils/wandb.py'),
                                   'polvo.utils.wandb.WandbRun.__enter__': ('utils.wandb.html#wandbrun.__enter__', 'polvo/utils/wandb.py'),
                                   'polvo.utils.wandb.WandbRun.__exit__': ('utils.wandb.html#wandbrun.__exit__', 'polvo/utils/wandb.py'),
                                   'polvo.utils.wandb.WandbRun.__init__': ('utils.wandb.html#wandbrun.__init__', 'polvo/utils/wandb.py'),
                                   'polvo.utils.wandb.wandb_upload': ('utils.wandb.html#wandb_upload', 'polvo/utils/wandb.py')},
            'polvo.visitor.core': { 'polvo.visitor.core.Visitor': ('visitor.core.html#visitor', 'polvo/visitor/core.py'),
                                    'polvo.visitor.core.Visitor._visit_bbox': ( 'visitor.core.html#visitor._visit_bbox',
                                                                                'polvo/visitor/core.py'),
                                    'polvo.visitor.core.Visitor._visit_bbox_labelled': ( 'visitor.core.html#visitor._visit_bbox_labelled',
                                                                                         'polvo/visitor/core.py'),
                                    'polvo.visitor.core.Visitor._visit_image_arr': ( 'visitor.core.html#visitor._visit_image_arr',
                                                                                     'polvo/visitor/core.py'),
                                    'polvo.visitor.core.Visitor._visit_image_file': ( 'visitor.core.html#visitor._visit_image_file',
                                                                                      'polvo/visitor/core.py'),
                                    'polvo.visitor.core.Visitor._visit_image_pil': ( 'visitor.core.html#visitor._visit_image_pil',
                                                                                     'polvo/visitor/core.py'),
                                    'polvo.visitor.core.Visitor._visit_keypoints': ( 'visitor.core.html#visitor._visit_keypoints',
                                                                                     'polvo/visitor/core.py'),
                                    'polvo.visitor.core.Visitor._visit_metadata': ( 'visitor.core.html#visitor._visit_metadata',
                                                                                    'polvo/visitor/core.py'),
                                    'polvo.visitor.core.Visitor._visit_obbox': ( 'visitor.core.html#visitor._visit_obbox',
                                                                                 'polvo/visitor/core.py'),
                                    'polvo.visitor.core.Visitor._visit_obbox_labelled': ( 'visitor.core.html#visitor._visit_obbox_labelled',
                                                                                          'polvo/visitor/core.py'),
                                    'polvo.visitor.core.Visitor._visit_segmask': ( 'visitor.core.html#visitor._visit_segmask',
                                                                                   'polvo/visitor/core.py'),
                                    'polvo.visitor.core.Visitor.collect_accept_fns': ( 'visitor.core.html#visitor.collect_accept_fns',
                                                                                       'polvo/visitor/core.py'),
                                    'polvo.visitor.core.Visitor.execute_accept_fns': ( 'visitor.core.html#visitor.execute_accept_fns',
                                                                                       'polvo/visitor/core.py'),
                                    'polvo.visitor.core.Visitor.visit_all': ( 'visitor.core.html#visitor.visit_all',
                                                                              'polvo/visitor/core.py'),
                                    'polvo.visitor.core.Visitor.visit_bbox': ( 'visitor.core.html#visitor.visit_bbox',
                                                                               'polvo/visitor/core.py'),
                                    'polvo.visitor.core.Visitor.visit_bbox_labelled': ( 'visitor.core.html#visitor.visit_bbox_labelled',
                                                                                        'polvo/visitor/core.py'),
                                    'polvo.visitor.core.Visitor.visit_image_arr': ( 'visitor.core.html#visitor.visit_image_arr',
                                                                                    'polvo/visitor/core.py'),
                                    'polvo.visitor.core.Visitor.visit_image_file': ( 'visitor.core.html#visitor.visit_image_file',
                                                                                     'polvo/visitor/core.py'),
                                    'polvo.visitor.core.Visitor.visit_image_pil': ( 'visitor.core.html#visitor.visit_image_pil',
                                                                                    'polvo/visitor/core.py'),
                                    'polvo.visitor.core.Visitor.visit_keypoints': ( 'visitor.core.html#visitor.visit_keypoints',
                                                                                    'polvo/visitor/core.py'),
                                    'polvo.visitor.core.Visitor.visit_metadata': ( 'visitor.core.html#visitor.visit_metadata',
                                                                                   'polvo/visitor/core.py'),
                                    'polvo.visitor.core.Visitor.visit_obbox': ( 'visitor.core.html#visitor.visit_obbox',
                                                                                'polvo/visitor/core.py'),
                                    'polvo.visitor.core.Visitor.visit_obbox_labelled': ( 'visitor.core.html#visitor.visit_obbox_labelled',
                                                                                         'polvo/visitor/core.py'),
                                    'polvo.visitor.core.Visitor.visit_segmask': ( 'visitor.core.html#visitor.visit_segmask',
                                                                                  'polvo/visitor/core.py')}}}



__version__ = "0.2.3"

import polvo.test

from polvo.utils import *
from polvo.data import *
from polvo.visitor import *
from polvo.draw import *

import polvo.datasets
import polvo.classification
import polvo.segmentation
import polvo.bbox
import polvo.keypoint
import polvo.segmask
import polvo.converter



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/10a_bbox.core.ipynb.

# %% auto 0
__all__ = ['BBox', 'OBBox', 'BBoxLabeled', 'OBBoxLabeled']

# %% ../../nbs/10a_bbox.core.ipynb 3
from fastcore.all import *
import math
import numpy as np
import polvo as pv

# %% ../../nbs/10a_bbox.core.ipynb 5
class _BBox:
    def __init__(self, points):
        self.points, _ = pv.sort_quadrilateral(np.array(points))
        
    def normalized_points(self, w, h): return self.points / np.array((w, h))
    
    @property
    def xyxyxyxy(self): return self.points
    @property
    def flat(self): return self.points.flatten()
    @property
    def xmin(self): return self.points[:, 0].min()
    @property
    def xmax(self): return self.points[:, 0].max()
    @property
    def ymin(self): return self.points[:, 1].min()
    @property
    def ymax(self): return self.points[:, 1].max()
    @property
    def width(self): return self.xmax-self.xmin
    @property
    def height(self): return self.ymax-self.ymin
    @property
    def area(self): return self.width*self.height
    @property
    def xyxy(self): return np.array((self.xmin, self.ymin, self.xmax, self.ymax))
    @property
    def yxyx(self): return np.array((self.ymin, self.xmin, self.ymax, self.xmax))
    @property
    def xywh(self): return np.array((self.xmin, self.ymin, self.width, self.height))
    @property
    def xycenter_wh(self): return np.array((self.xmin+self.width/2, self.ymin+self.height/2, self.width, self.height))
        
    @classmethod
    def from_points(cls, points): return cls(points)
    @classmethod
    def from_flat(cls, points): return cls(np.reshape(points, (4, 2)))

# %% ../../nbs/10a_bbox.core.ipynb 6
class BBox(_BBox):
    """Bounding Box representation.
    Should **not** be instantiated directly, instead use `from_*` methods. e.g. `from_xyxy`, `from_xywh`.
    Is able to transform coordinates into different formats, e.g. `xyxy`, `xywh`.
    
    # Examples
    Create from `xywh` format, and get `xyxy` coordinates.
    ```python
    bbox = BBox.from_xywh(1, 1, 4, 4)
    xyxy = bbox.xyxy
    ```
    """
    def __repr__(self):
        return f"<{self.__class__.__name__} (xmin:{self.xmin}, ymin:{self.ymin}, xmax:{self.xmax}, ymax:{self.ymax})>"

    def __eq__(self, other) -> bool:
        if isinstance(other, BBox): return self.xyxyxyxy == other.xyxyxyxy
        return False
    
    def accept_visit(self, visitor): return visitor.visit_bbox(self)

    def relative_xcycwh(self, img_width: int, img_height: int):
        scale = np.array([img_width, img_height, img_width, img_height])
        x, y, w, h = self.xywh / scale
        xc = x + 0.5*w
        yc = y + 0.5*h
        return (xc, yc, w, h)
    
    def normalized_xycenter_wh(self, img_width: int, img_height: int):
        scale = np.array([img_width, img_height, img_width, img_height])
        return self.xycenter_wh / scale

    @classmethod
    def from_xyxy(cls, xmin, ymin, xmax, ymax): return cls.from_flat((xmin,ymin, xmax,ymin, xmax,ymax, xmin,ymax))
    @classmethod
    def from_xywh(cls, x, y, w, h): return cls.from_xyxy(x, y, x+w, y+h)
    @classmethod
    def from_xycenter_wh(cls, xc, yc, w, h):
        return cls.from_xywh(xc-w/2, yc-h/2, w, h)
    @classmethod
    def from_relative_xcycwh(cls, xc, yc, bw, bh, img_width, img_height):
        # subtracting 0.5 goes from center to left/upper edge, adding goes to right/bottom
        pnts = [(xc - 0.5*bw), (yc - 0.5*bh), (xc + 0.5*bw), (yc + 0.5*bh)]
        # convert from relative to absolute coordinates
        scale = np.array([img_width, img_height, img_width, img_height])
        xmin, ymin, xmax, ymax = np.around(pnts * scale).astype(int).tolist()
        return cls.from_xyxy(xmin, ymin, xmax, ymax)
    @classmethod
    def from_rle(cls, rle, h, w):
        a = np.array(rle.counts, dtype=int)
        a = a.reshape((-1, 2))  # an array of (start, length) pairs
        a[:, 0] -= 1  # `start` is 1-indexed
        y0 = a[:, 0] % h
        y1 = y0 + a[:, 1]
        if np.any(y1 > h):
            # got `y` overrun, meaning that there are a pixels in mask on 0 and shape[0] position
            y0 = 0
            y1 = h
        else:
            y0 = np.min(y0)
            y1 = np.max(y1)
        x0 = a[:, 0] // h
        x1 = (a[:, 0] + a[:, 1]) // h
        x0 = np.min(x0)
        x1 = np.max(x1)
        if x1 > w:
            # just went out of the image dimensions
            raise ValueError(f"invalid RLE or image dimensions: x1={x1} > shape[1]={w}")
        return cls.from_xyxy(x0, y0, x1, y1)

# %% ../../nbs/10a_bbox.core.ipynb 12
class OBBox(_BBox):
    @classmethod
    def from_clockwise(cls, x,y, w,h, degrees):
        r = math.radians(degrees)
        sinr, cosr = math.sin(r), math.cos(r)
        x2 = x+cosr*w
        y2 = y+sinr*w
        x3 = x+cosr*w-sinr*h
        y3 = y+sinr*w+cosr*h
        x4 = x-sinr*h
        y4 = y+cosr*h
        
        return cls.from_flat((x,y, x2,y2, x3,y3, x4,y4))
    
    def accept_visit(self, visitor): return visitor.visit_obbox(self)
    
    # TODO: when angle conversion is implemented, use it here
    def __repr__(self): return f"<{self.__class__.__name__} ({self.points})>"

# %% ../../nbs/10a_bbox.core.ipynb 15
class _BBoxLabeled(GetAttr):
    _default='bbox'
    def __init__(self, bbox, label): store_attr()
    def __repr__(self): return f'{self.label} | {self.bbox}'

# %% ../../nbs/10a_bbox.core.ipynb 16
class BBoxLabeled(_BBoxLabeled):
    def accept_visit(self, visitor): return visitor.visit_bbox_labelled(self)

# %% ../../nbs/10a_bbox.core.ipynb 18
class OBBoxLabeled(_BBoxLabeled):
    def accept_visit(self, visitor): return visitor.visit_obbox_labelled(self)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/10e_keypoints.vis.ipynb.

# %% auto 0
__all__ = ['overlay']

# %% ../../nbs/10e_keypoints.vis.ipynb 3
from fastcore.all import *
import PIL
import numpy as np
import polvo as pv
# from typing import Tuple
# from PIL import Image, ImageFont
from PIL.ImageDraw import ImageDraw
# from polvo.bbox import BBox, OBBox, BBoxLabeled, OBBoxLabeled
from . import Keypoints

# %% ../../nbs/10e_keypoints.vis.ipynb 4
def overlay(image:PIL.Image, keypoints:Keypoints, color:Tuple[int,int,int]=None, dot_radius=None):
    """Draws a box on an image with a given color.
    # Arguments
        image     : The image to draw on.
        box       : A list of 4 elements (x1, y1, x2, y2).
        color     : The color of the box.
    """
    draw = PIL.ImageDraw.Draw(image)
    color = color or (255, 50, 50)
    
    r = dot_radius or int(min(image.size)/100)
    for i, (x,y) in enumerate(keypoints.xy_points):
        coords = x-r,y-r,x+r,y+r
        draw.ellipse(coords, fill=color)
        
    return image
